.multi-wrapper {
    border: 1px solid #ccc;
    border-radius: 3px;
    width: 100%;
}

.multi-wrapper .non-selected-wrapper,
.multi-wrapper .selected-wrapper {
    box-sizing: border-box;
    display: inline-block;
    height: 200px;
    overflow-y: scroll;
    padding: 10px;
    vertical-align: top;
    width: 50%;
}

.multi-wrapper .non-selected-wrapper {
    background: #fafafa;
    border-right: 1px solid #ccc;
}

.multi-wrapper .selected-wrapper {
    background: #fff;
}

.multi-wrapper .item {
    cursor: pointer;
    display: block;
    padding: 5px 10px;
}

.multi-wrapper .item:hover {
    /* background: #ececec; */ /* by Gioni */
    background-color: #ececec; /* by Gioni */
    border-radius: 2px;
}

.multi-wrapper .search-input {
    border: 0;
    border-bottom: 1px solid #ccc;
    border-radius: 0;
    display: block;
    font-size: 1em;
    margin: 0;
    outline: 0;
    padding: 10px 20px;
    width: 100%;
    box-sizing: border-box;
}

.multi-wrapper .non-selected-wrapper .item.selected {
    opacity: 0.5;
}

.multi-wrapper .non-selected-wrapper .item.disabled,
.multi-wrapper .selected-wrapper .item.disabled {
  opacity: 0.5;
  text-decoration: line-through;
}

.multi-wrapper .non-selected-wrapper .item.disabled:hover,
.multi-wrapper .selected-wrapper .item.disabled:hover {
  background: inherit;
  cursor: inherit;
}
