@font-face {
  font-family: 'crb';
  src:
    url('fonts/crb.ttf?v388dd') format('truetype'),
    url('fonts/crb.woff?v388dd') format('woff'),
    url('fonts/crb.svg?v388dd#crb') format('svg');
  font-weight: normal;
  font-style: normal;
  font-display: block;
}

.crb-icon {
  /* use !important to prevent issues with browser extensions that change fonts */
  font-family: 'crb' !important;
  speak: never;
  font-style: normal;
  font-weight: normal;
  font-variant: normal;
  text-transform: none;
  line-height: 1;

  /* Better Font Rendering =========== */
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.crb-icon-bxl-google:before {
  content: "\e908";
}
.crb-icon-bx-bell:before {
  content: "\e200";
}
.crb-icon-bx-bell-off:before {
  content: "\e937";
}
.crb-icon-bx-bell-plus:before {
  content: "\e938";
}
.crb-icon-bx-bolt:before {
  content: "\e93d";
}
.crb-icon-bx-briefcase-alt:before {
  content: "\e201";
}
.crb-icon-bx-bug:before {
  content: "\e949";
}
.crb-icon-bx-bulb:before {
  content: "\e202";
}
.crb-icon-bx-chip:before {
  content: "\e970";
}
.crb-icon-bx-cog:before {
  content: "\e203";
}
.crb-icon-bx-collection:before {
  content: "\e980";
}
.crb-icon-bx-dashboard:before {
  content: "\e204";
}
.crb-icon-bx-download:before {
  content: "\e9a3";
}
.crb-icon-bx-error:before {
  content: "\e205";
}
.crb-icon-bx-error-circle:before {
  content: "\e206";
}
.crb-icon-bx-flask:before {
  content: "\e9c2";
}
.crb-icon-bx-group:before {
  content: "\e207";
}
.crb-icon-bx-hide:before {
  content: "\e9db";
}
.crb-icon-bx-history:before {
  content: "\e9dc";
}
.crb-icon-bx-idea:before {
  content: "\e208";
}
.crb-icon-bx-info-circle:before {
  content: "\e9e9";
}
.crb-icon-bx-key:before {
  content: "\e209";
}
.crb-icon-bx-layer:before {
  content: "\e210";
}
.crb-icon-bx-list-check:before {
  content: "\e9ff";
}
.crb-icon-bx-lock:before {
  content: "\e211";
}
.crb-icon-bx-pulse:before {
  content: "\e212";
}
.crb-icon-bx-radar:before {
  content: "\e213";
}
.crb-icon-bx-search:before {
  content: "\ea5c";
}
.crb-icon-bx-shield:before {
  content: "\e214";
}
.crb-icon-bx-shield-alt:before {
  content: "\e215";
}
.crb-icon-bx-show:before {
  content: "\e216";
}
.crb-icon-bx-slider:before {
  content: "\e217";
}
.crb-icon-bx-slider-alt:before {
  content: "\e218";
}
.crb-icon-bx-time:before {
  content: "\e219";
}
.crb-icon-bx-trash:before {
  content: "\ea98";
}
.crb-icon-bx-umbrella:before {
  content: "\eaa1";
}
.crb-icon-bx-user-detail:before {
  content: "\eaac";
}
.crb-icon-bx-wrench:before {
  content: "\e220";
}
.crb-icon-bxs-archive-out:before {
  content: "\eacf";
}
.crb-icon-bxs-bell:before {
  content: "\e221";
}
.crb-icon-bxs-bell-off:before {
  content: "\ead9";
}
.crb-icon-bxs-dashboard:before {
  content: "\e222";
}
.crb-icon-bxs-down-arrow-circle:before {
  content: "\eb1c";
}
.crb-icon-bxs-error:before {
  content: "\eb25";
}
.crb-icon-bxs-error-circle:before {
  content: "\e223";
}
.crb-icon-bxs-info-circle:before {
  content: "\eb44";
}
.crb-icon-bxs-rocket:before {
  content: "\e224";
}
.crb-icon-bxs-shield:before {
  content: "\e225";
}
.crb-icon-bxs-shield-alt:before {
  content: "\e226";
}
.crb-icon-bxs-slider-alt:before {
  content: "\e227";
}
.crb-icon-bxs-world:before {
  content: "\e228";
}
