/* WP menu if on remote */
.crb-remote .wp-not-current-submenu .wp-menu-name,
.crb-remote .wp-not-current-submenu .wp-menu-image::before {
    color: #555 !important;
}

.crb-remote .wp-not-current-submenu .update-plugins,
.crb-remote .wp-not-current-submenu .awaiting-mod {
    display: none !important;
}

/* Admin bar if on remote */
.crb-remote #wpadminbar,
.crb-remote #wpadminbar div.ab-sub-wrapper,
.crb-remote #wpadminbar div.ab-sub-wrapper li,
.crb-remote #wpadminbar .ab-item {
    /*background-color: #2d5c8b !important;*/
    background-color: #005696 !important;
    color: #fff !important;
}

#wp-admin-bar-crb_site_switch .ab-sub-wrapper .ab-item:hover {
    color: #fff !important;
    background-color: #777 !important;
}

.crb-remote #wp-admin-bar-crb_site_switch .ab-sub-wrapper .ab-item:hover {
    color: #fff !important;
    background-color: #333 !important;
}

ul li#wp-admin-bar-crb_site_switch > a {
    letter-spacing: 0.06em !important;
}

ul li#wp-admin-bar-crb_site_switch {
    padding-right: 1em;
}

#wp-admin-bar-crb_site_switch .ab-icon::before {
    content: "\f333";
}

#wp-admin-bar-crb_site_switch div.ab-sub-wrapper {
    max-height: 500px;
    max-width: 500px;
    min-width: 220px !important;
    /*padding-right: 2em !important;*/
    overflow: auto;
    /*overflow-x: visible;*/
}

#wpadminbar .ab-icon {
    top: 2px !important;
}

.crb-remote #wpadminbar .ab-icon::before {
    color: #fff;
}

#wp-admin-bar-crb_slave_site_menu .ab-icon::before {
    content: "\f102";
}

#wp-admin-bar-crb_to_master .ab-icon::before {
    content: "\f158";
}

@media screen and (min-width: 1000px) {
    #crb-form-slave-edit-form table {
        max-width: 800px;
        /*padding-right: 30%;*/
    }
}

/* Site List */

#crb-nexus-sites table td {
    overflow: hidden;
}

@media screen and (min-width: 768px) {
    .crb-main .fixed .column-site_name {
        width: 20%;
    }

    .crb-main .fixed .column-wp_v,
    .crb-main .fixed .column-plugin_v,
    .crb-main .fixed .column-updates {
        width: 8%;
    }
}

.crb-main .column-site_url {
    width: 18%;
}

.crb-main .column-site_url,
.crb-main .column-site_url a {
    /*overflow-wrap: normal;*/
    /*word-break: keep-all !important; */
    word-wrap: normal !important;
}

#crb-nexus-sites .tablenav select,
#crb-nexus-sites .tablenav input[type="text"] {
    width: auto;
}

#crb-nexus-sites .bulkactions select option:last-child {
    color: #777;
}

#crb-nexus-sites #crb-top-filter {
    float: left;
    margin-right: 3em;
}

.column-primary .crb-slave-url {
    color: #555;
}

.crb-slave-ver {
    margin-right: 0.5em;
}

.crb-vpro {
    display: inline-block;
    color: #fff;
    background-color: #38ab6b;
    padding: 0em 0.4em 0em 0.4em;
    font-weight: bold;
    font-size: 90%;
    text-align: center;
    border-radius: 3px;
}